/*
 * Name:    mg_outh.c
 *
 * Purpose: Output handlers for the MX message log window.
 *
 * Author:  William Lavender
 *
 *-----------------------------------------------------------------------
 *
 * Copyright 1999-2000 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <string.h>

#include <tcl.h>
#include <tk.h>

#include "mx_util.h"
#include "mx_rec.h"

#include "mx_tcl.h"

#define MXGUI_BLACK	0
#define MXGUI_RED	1
#define MXGUI_ORANGE	2
#define MXGUI_BLUE	3

static void Mxgui_info_output_handler( char * );
static void Mxgui_warning_output_handler( char * );
static void Mxgui_error_output_handler( char * );
static void Mxgui_debug_output_handler( char * );

static void Mxgui_info_dialog_handler( char *, char *, char * );

static void Mxgui_generic_output_handler( int, char * );

static Tcl_Interp *Mxgui_generic_output_interp;

static Tk_Window Mxgui_generic_output_window;

static char Mxgui_generic_output_window_name[80];

static Tcl_Interp *Mxgui_generic_dialog_interp;

MX_EXPORT int
Mx_install_output_handlers_cmd( ClientData clientData, Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[] )
{
	const char fname[] = "Mx_install_output_handlers_cmd()";

	Tk_Window main_window;
	char *window_name;
	mx_status_type mx_status;

	if ( objc != 2 ) {
		Tcl_WrongNumArgs( interp, 1, objv,
		"Usage: install_output_handlers window" );
		return TCL_ERROR;
	}

	window_name = Tcl_GetStringFromObj( objv[1], NULL );

	if ( window_name == NULL ) {
		mx_status = mx_error( MXE_ILLEGAL_ARGUMENT, fname,
	"Second argument to Mx::install_output_handlers was not a string." );

		return mxtcl_error( interp, mx_status );
	}

	mx_strncpy( Mxgui_generic_output_window_name, window_name,
				sizeof( Mxgui_generic_output_window_name ) );

	Mxgui_generic_output_interp = interp;

	main_window = Tk_MainWindow( interp );

	if ( main_window == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, fname,
"Tk does not appear to be loaded since the Tk main window does not exist." );

		return mxtcl_error( interp, mx_status );
	}

	Mxgui_generic_output_window
		= Tk_NameToWindow( interp, window_name, main_window );

	if ( Mxgui_generic_output_window == NULL ) {
		mx_status = mx_error( MXE_FUNCTION_FAILED, fname,
		"The MX gui message log window does not seem to exist." );

		return mxtcl_error( interp, mx_status );
	}

	mx_set_info_output_function( Mxgui_info_output_handler );
	mx_set_warning_output_function( Mxgui_warning_output_handler );
	mx_set_error_output_function( Mxgui_error_output_handler );
	mx_set_debug_output_function( Mxgui_debug_output_handler );

	return TCL_OK;
}

MX_EXPORT int
Mx_install_dialog_handler_cmd( ClientData clientData, Tcl_Interp *interp,
				int objc, Tcl_Obj *CONST objv[] )
{
	Mxgui_generic_dialog_interp = interp;

	mx_set_info_dialog_function( Mxgui_info_dialog_handler );

	return TCL_OK;
}

static void
Mxgui_info_output_handler( char *output )
{
	Mxgui_generic_output_handler( MXGUI_BLACK, output );

	return;
}

static void
Mxgui_warning_output_handler( char *output )
{
	Mxgui_generic_output_handler( MXGUI_ORANGE, output );

	return;
}

static void
Mxgui_error_output_handler( char *output )
{
	Mxgui_generic_output_handler( MXGUI_RED, output );

	return;
}

static void
Mxgui_debug_output_handler( char *output )
{
	Mxgui_generic_output_handler( MXGUI_BLUE, output );

	return;
}

static void
Mxgui_generic_output_handler( int color, char *output )
{
	char command_format[] =
		"set mxprivate_textwindow [ %s childsite ]; "
		"$mxprivate_textwindow configure -state normal; "
		"%s insert end \"%s\n\"; "
		"$mxprivate_textwindow configure -state disabled; "
		"%s yview moveto 1.0";

	char command[1024];

	/* There's got to be a more efficient way to add the text to the 
	 * message log window than this, but I haven't found it yet.
	 */

	sprintf( command, command_format,
		Mxgui_generic_output_window_name,
		Mxgui_generic_output_window_name, output,
		Mxgui_generic_output_window_name );

	(void) Tcl_Eval( Mxgui_generic_output_interp, command );

	/* Allow some Tcl events to be processed. */

	mxtcl_process_tcl_events( 100 );

	return;
}

static void
Mxgui_info_dialog_handler( char *text_prompt, char *gui_prompt,
					char *button_text )
{
	char command[400];

	sprintf( command,
	  "bell; tk_dialog .mx_info_dialog \"MX Info\" \"%s\" info 0 \"%s\"",
		gui_prompt, button_text );

	(void) Tcl_Eval( Mxgui_generic_dialog_interp, command );

	/* Allow some Tcl events to be processed. */

#if 0
	mxtcl_process_tcl_events( 100 );
#endif

	return;
}

